from OFS.SimpleItem import SimpleItem
from zope.interface import implements
from interfaces import IDragCartContent
from Products.jsonserver.minjson import write as json_write

class DictObj(dict):
    
    def __init__(self, **kw):
        self.update(kw)

class DragCartContent(SimpleItem):
    implements(IDragCartContent)
    
    meta_type = 'json_examples Drag Cart Content'

    def __init__(self, id, title):
        self.id = id
        self.title = title
        self.cart_content = ()

    _data = [
            DictObj(id='21345', name='take this', price=12.00, image='1.png'),
            DictObj(id='132x', name='take that', price=20.50, image='2.png'),
            DictObj(id='21348', name='real solution', price=5.35, image='3.png'),
            DictObj(id='X-442', name='our best offer', price=112.10, image='4.png'),
            ]

    _data_by_id = {}
    for row in _data:
        _data_by_id[row['id']] = row

    def getGoodiesData(self):
        '''Get the goodies data

        in a dict name, price, pict
        '''
        return self._data
            
    def getGoodieById(self, id):
        '''Get a goodie by id

        or None if the id is invalid.
        '''
        try:
            return self._data_by_id[id]
        except KeyError:
            return None
            
    def manageCartContent(self, cart_content=None, autosave=False, add_to_cart=None, delete_index_from_cart=None,
            save=False, revert=False, **kw):
        '''manage cart content

        Helps the template to calculate the new shopping cart
        contents.
        
        cart_content: the cart content from the request (client state)
        delete_index_from_cart: cart indices to delete.
        add_to_cart: goodie ids to add
        save: to save the value
        revert: to revert to the previously saved value
        '''
        def listify(obj):
            if obj is None:
                return ()
            elif not isinstance(obj, (tuple, list)):
                return (obj, )
            else:
                return obj
        if revert or cart_content is None:
            cart_content = self.cart_content
        add_to_cart = listify(add_to_cart)
        delete_index_from_cart = listify(delete_index_from_cart)
        new_cart_content = [id for index, id in enumerate(cart_content) if str(index) not in delete_index_from_cart]
        new_cart_content.extend(add_to_cart)
        if save or autosave:
            self.cart_content = tuple(new_cart_content)
        return new_cart_content
        
    def json_write(self, obj, encoding='utf-8', outputEncoding='utf-8'):
        return json_write(obj, encoding, outputEncoding)
        
    def sumGoodie(self, sums, goodie):
        sums['price'] += goodie['price']

    def formatPrice(self, price):
        return '%.2f &euro;' % price

# Product registration

from Products.PageTemplates.PageTemplateFile import PageTemplateFile

manage_addDragCartContentForm = PageTemplateFile(
    "browser/content_add.pt", globals(),
    __name__ = 'manage_addDragCartContentForm')

def manage_addDragCartContent(self, id, title, REQUEST=None):
    """Add Drag Cart content."""
    id = self._setObject(id,DragCartContent(id, title))
    if REQUEST is None:
        return
    REQUEST.RESPONSE.redirect(REQUEST['URL1'] + '/manage_main')

def initialize(context):
    context.registerClass(
        DragCartContent,
        constructors = (manage_addDragCartContentForm,
                        manage_addDragCartContent),
        )
