import docutils.io
from Products.Five import BrowserView
try:
    from Products.Five.browserconfigure import page
except ImportError:
    # No idea which import path is the canonical one.
    # This is needed for Five installed for Zope2.7.
    from Products.Five.browser.metaconfigure import page

class Warnings:
    def __init__(self):
        self.messages = []
    def write(self, message):
        self.messages.append(message)

class DisplayHelp(BrowserView):

    # default attrs that can be overwritten on the context
    report_level = 2
    stylesheet = 'default.css'
    input_encoding = 'utf-8'
    output_encoding = 'utf-8'
    source_path = None
    
    def __call__(self):
        ' Render the help text'

        # format with strings
        pub = docutils.core.Publisher()
        pub.set_reader('standalone', None, 'restructuredtext')
        pub.set_writer('html')

        # go with the defaults
        pub.get_settings()

        # this is needed, but doesn't seem to do anything
        pub.settings._destination = ''

        # use the stylesheet chosen by the user
        pub.settings.stylesheet = self.stylesheet

        # set the reporting level to something sane
        pub.settings.report_level = int(self.report_level)

        # disallow use of the .. include directive for security reasons
        pub.settings.file_insertion_enabled = 0

        # don't break if we get errors
        pub.settings.halt_level = 6

        # remember warnings
        pub.settings.warning_stream = Warnings()

        pub.source = docutils.io.FileInput(
            source_path = self.source_path, 
            encoding = self.input_encoding,
            handle_io_errors = False,
            )

        # output - not that it's needed
        pub.settings.output_encoding = self.output_encoding
        pub.destination = docutils.io.StringOutput(
            encoding = self.output_encoding,
            )

        # parse!
        document = pub.reader.read(pub.source, pub.parser, pub.settings)

        # transform
        pub.apply_transforms(document)

        #_warnings = ''.join(pub.settings.warning_stream.messages)

        # do the format
        _formatted = pub.writer.write(document, pub.destination)

        return _formatted
 
def display_help(_context, name, for_, source_path, stylesheet='default.css', layer='default', permission='zope.Public'):

    source_path = _context.path(source_path)
    
    try:
        displayHelpClass = type(
            str("Display help page %s for %s (source_path %s, stylesheet %s)" %(name, for_, source_path, stylesheet)),
            (DisplayHelp, ), {
                'source_path': source_path,
                'stylesheet': stylesheet,
            })
    except TypeError:
        # Huh. For Zope2.7, due to inheritence from old-style ExtensionClass, the above does not work.
        # XXX Have not yet decided about the best way to resolve this.
        class displayHelpClass(DisplayHelp):
            pass
        displayHelpClass.source_path = source_path
        displayHelpClass.stylesheet = stylesheet

    page(_context, name, permission, for_, layer, class_=displayHelpClass)
